function [En,psin,rhon]=Schrodinger(xmin,xmax,Ep,m,num_sol)
% Schrodinger resuelve la ecuacin de Schrdinger para determinar sus 
%  valores proprios y sus vectores propios, es decir las energas de los estados 
%  proprios (estacionario). 
%  Sirve en particular para los estados ligados (confinados) puesto que en ciertos casos
%  el mtodo de la matriz de transferencia es demasiado sensible a las aproximaciones numricas.
%
%   [En,psin,rhon]=Schrodinger(xmin,xmax,Ep,m,num_sol)
%
%   	>> xmin 	= 	Lmite inferior en el eje Ox (nm)
%   	>> xmax 	= 	Lmite superior en el eje Ox (nm)
%   	>> Ep		=	Energa potencial (eV)
%   	>> m 		=	Masa del objeto fsico (respecto a la del electrn)
%   	>> num_sol	=	Nmero de soluciones (energas) deseadas
%		<< En		=	Energa del estado ligado (eV)
%		<< psin		= 	Funcin de onda correspondiente a En
%		<< rhon		=  	Densidad lineal de probabilidad correspondiente a En
%
% Olivier Pujol (2010)   
%
%% Constantes y conversiones

h=6.6260755E-34;    % Constante de Planck (J.s)
hbar=h/(2*pi);
hbar2=hbar^2;
me=0.91093897E-30;  % Masa del electrn (kg)
eV=1.60217733E-19;  % Electronvolt (J) 
nm=1E-9;            % Nanometro (m)

m=m*me;             % Conversion de la masa

n=length(Ep);       % Nmero de puntos 
incremento=(xmax-xmin)/n;  % Incremento de clculo (nm)

cte=(hbar2)/(2*m*eV*(nm*incremento)^2);    % Constante que aparece en la ecuacin de Schrdinger 

clear hbar hbar2 m me eV nm

%% Matriz hamiltoniana

dia=nan(1,n-1);                     % Inicializacin de los elementos diagonales 
nodia=nan(1,n-1);                  % Inicializacin de los elementos no diagonales 

for i=2:n
    if min(Ep)<0
        dia(i-1)=2*cte+Ep(i)-min(Ep);           % Elemento diagonal
    else
        dia(i-1)=2*cte+Ep(i);           		% Elemento diagonal
    end
    nodia(i-1)=cte;                			% Elemento no diagonal
end

d1=-nodia(2:n-1);							
Hamilt=diag(d1,1);                  % Diagonal superior del hamiltoniano							
Hamilt=Hamilt+diag(d1,-1);          % Diagonal inferior del hamiltoniano

d2=dia(1:n-1);
Hamilt=Hamilt+diag(d2,0);           % Diagonal del hamiltoniano

clear i cte dia nodia d1 d2 

%% Resolucin de la ecuacin estacionaria H\psi = E\psi

[psi,E]=eigs(Hamilt,num_sol,'SM');			% Se usa la funcin de Matlab eigs para determinar los estados propios

clear Hamilt

% Valores propios
En=nan(1,num_sol);
for i=1:num_sol
    En(i)=E(i,i)+min(Ep);
end

clear E

% Vectores propios
psin=[zeros(1,num_sol);psi;zeros(1,num_sol)];  % La funcin de onda es cero en x = 0 y x = xmax

clear psi

%% Clculo de la densidad de probabilidad normalizada 

rhon=abs(psin).^2;

% normalizacin
aire=nan(1,size(rhon,2));
for in=1:size(rhon,2)
    rea(in)=sum((rhon(1:end-1,in)+rhon(2:end,in))*incremento)/2;    
    rhon(:,in)=rhon(:,in)/rea(in);
    psin(:,in)=psin(:,in)/sqrt(rea(in));
end

clear rea in incremento

%% Grfico

nt=input('Nmero de representacin grfica (1 <= nt <= 7)?');

x=linspace(xmin,xmax,n);

if (nt>num_sol),error('El nmero considerado es ms grande que el nmero de valores propios!');end
if (nt<=0),error('El nmero considerado tiene que ser al menos 1!');end
if (nt>=7),error('El nmero considerado tiene que ser al mximo 7!');end

for nfig=1:nt
    figure(nfig)
    subplot(2,1,1)
    plot(x,Ep);
    if (min(Ep)<0),axis([xmin xmax min(Ep) 0]);end
    if (min(Ep)>=0),axis([xmin xmax min(Ep) Ep(end)]);end
    set(gca,'xminortick','on','yminortick','on');
    grid on;
    box on
    xlabel('{\itx} (nm)');
    title('{Energa potencial (eV)}');
	%
    subplot(2,1,2)
    plot(x,rhon(1:end-1,nt-nfig+1),'r');
    set(gca,'xminortick','on','yminortick','on');
    grid on;
    box on
    axis tight
    xlabel('{\itx} (nm)');
    ylabel(['{|\it\psi_',num2str(nfig),' |^{2}}']);
end

clear ifig

if (nt==1),return;end

% Leyenda para el grfico siguiente 

Leg=cell(1,nt);
for il=1:nt
    Leg{il}=['{|\it\psi_',num2str(il),' |^{2}}'];
end
clear il
%
figure
subplot(2,1,1)
plot(x,Ep);
if (min(Ep)<0),axis([xmin xmax min(Ep) 0]);end
if (min(Ep)>=0),axis([xmin xmax min(Ep) Ep(end)]);end
set(gca,'xminortick','on','yminortick','on');
grid on;
box on
xlabel('{\itx} (nm)');
title('{Energa potencial (eV)}');
%
subplot(2,1,2)
C=['r','b','k','g','m','y','c'];
for it=1:nt     
    plot(x,rhon(1:end-1,nt-it+1),C(it));
    hold on
end
legend(Leg)
set(gca,'xminortick','on','yminortick','on');
grid on;
box on
axis tight
xlabel('{\itx} (nm)');
ylabel('{|\it\psi |^{2}}');

return